#!/usr/bin/env python3

import certifi
import os
import pytest
import requests
import socket
import shutil
import sys
from OpenSSL import SSL


def add_custom_cert_authorities(ca_file: str,
                                custom_ca_files: list,
                                dest_file: str =
                                '/tmp/custom_ca_bundle.crt'):
    """Concatenates existing cert bundle with custom CAs."""

    destination = open(dest_file, 'wb')
    with open(dest_file, 'wb') as destination, open(ca_file, 'rb') as ca:
        shutil.copyfileobj(ca, destination)
        for custom_ca_file in custom_ca_files:
            with open(custom_ca_file, 'rb') as custom_ca:
                shutil.copyfileobj(custom_ca, destination)


def fetch_certs(domain: str, port: int = 443):
    """Fetches cert fom given domain."""

    s = socket.socket()
    context = SSL.Context(SSL.TLSv1_2_METHOD)
    conn = SSL.Connection(context, s)
    conn.set_tlsext_host_name(domain.encode('utf-8'))
    certs = []

    try:
        conn.connect((domain, port))
    except socket.gaierror as e:
        print('socket.gaierror: {0}'.format(str(e)))
        sys.exit(1)

    conn.do_handshake()
    try:
        certs = conn.get_peer_cert_chain()
    except SSL.Error as e:
        print('SSL Error: {0}'.format(str(e)))
        sys.exit(1)

    return certs


def print_cert_info(certs: list):
    """Show CN and issuer CN of cert list."""

    for cert in certs:
        subject = cert.get_subject()
        cn = subject.CN
        issuer = cert.get_issuer().CN
        print('CN: {0} (Issuer: {1})'.format(cn, issuer))


def read_certs_from_file(filename: str):
    """Read cert from file for debugging/development."""

    import OpenSSL.crypto
    cert = OpenSSL.crypto.load_certificate(
        OpenSSL.crypto.FILETYPE_PEM,
        open(filename).read()
    )
    return [cert]


def valid_cert(domain: str, ca_file: str = '/tmp/custom_ca_bundle.crt'):
    """Validate cert of given domain against a ca_file bundle."""

    valid = False

    url = 'https://' + domain
    print('Validating cert from {0} ...'.format(url))
    certs = fetch_certs(domain)
    print_cert_info(certs)

    try:
        requests.get(url, verify=ca_file)
    except requests.exceptions.SSLError as ex:
        print('SSL Verification Error {}'.format(ex))
        for cert in certs:
            issuer = cert.get_issuer().CN
            if issuer == 'cert-manager.local':
                print('Allowing exception for self-signed cert-mananger cert.')
                valid = True
        return valid

    print('Successfully Verified SSL Cert.\n')
    return True


@pytest.mark.certs
def test_cert_validation(host):
    """Checks for proper cluster certs from exposed services.
    Check is executed on the local provisioning machine.
    """

    # Use FQDN env var if set, otherwise use domain var from
    # settings.yml.
    domain = os.environ.get("FQDN")
    if domain:
        print("Using domain %s from FQDN environment variable." % domain)
    else:
        ansible_vars = host.ansible.get_variables()
        domain = ansible_vars["domain"]
        print("Using domain %s from ansible settings.yml." % domain)

    add_custom_cert_authorities(certifi.where(),
                                ['pytest/le-staging-bundle.pem'])

    assert valid_cert('chat.{0}'.format(domain))
    assert valid_cert('files.{0}'.format(domain))
    assert valid_cert('office.{0}'.format(domain))
    assert valid_cert('grafana.{0}'.format(domain))


if __name__ == "__main__":
    test_cert_validation('')
